/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.imageloader

import android.graphics.Bitmap
import android.os.Looper
import android.widget.ImageView
import androidx.core.content.ContextCompat
import com.kakao.sdk.partner.friend.imageloader.ImageLoader.Companion.drawBitmap
import com.kakao.sdk.partner.friend.imageloader.network.ConvertBitmapAction
import com.kakao.sdk.partner.friend.imageloader.network.ConvertMultiBitmapAction
import com.kakao.sdk.partner.friend.imageloader.network.ImageViewAction

/**
 * @suppress
 */
internal class ActionCreator(
    private val loader: ImageLoader,
    private val imageUrls: List<String?>,
    private val downloader: ImageDownloader
) {
    private var placeHolder: Int = Constants.NO_PLACEHOLDER

    fun placeHolder(resourceId: Int): ActionCreator {
        this.placeHolder = resourceId
        return this
    }

    fun into(target: ImageView) {
        if (Looper.getMainLooper().thread != Thread.currentThread()) {
            throw IllegalStateException("Method call should happen from the main thread.")
        }

        if (placeHolder != Constants.NO_PLACEHOLDER) {
            target.setImageDrawable(ContextCompat.getDrawable(target.context, placeHolder))
        }

        imageUrls[0]?.let {
            val bitmap = ImageLoader.loadCache(it)
            bitmap?.let {
                drawBitmap(target, it)
                return
            }
            loader.submit(ImageViewAction(target, it, downloader))
        }
    }

    fun toBitmap(callback: (bitmap: Bitmap?) -> Unit) {
        if (Looper.getMainLooper().thread != Thread.currentThread()) {
            throw IllegalStateException("Method call should happen from the main thread.")
        }

        imageUrls[0]?.let {
            val bitmap = ImageLoader.loadCache(it)
            bitmap?.let {
                ImageLoader.getBitmap(bitmap, callback)
                return
            }
            loader.submit(ConvertBitmapAction(it, downloader, callback))
        }
    }

    fun toMultiBitmap(callback: (List<Bitmap>?) -> Unit) {
        if (Looper.getMainLooper().thread != Thread.currentThread()) {
            throw IllegalStateException("Method call should happen from the main thread.")
        }

        if (imageUrls == null) {
            return
        }
        loader.submit(ConvertMultiBitmapAction(imageUrls, downloader, callback))
    }
}