/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.view

import android.app.Activity
import android.os.Bundle
import android.os.Parcelable
import android.os.ResultReceiver
import androidx.appcompat.app.AppCompatActivity
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.friend.Constants
import com.kakao.sdk.partner.friend.Repository
import com.kakao.sdk.partner.friend.util.LayoutUtils

/**
 * @suppress
 */
internal abstract class PickerActivity : AppCompatActivity() {
    private lateinit var resultReceiver: ResultReceiver

    abstract fun setViewAppearance(isPopup: Boolean, param: Parcelable)
    abstract fun setViewOrientation(param: Parcelable)
    abstract fun setFragment()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        try {
            intent.extras?.getBundle(Constants.KEY_BUNDLE)?.run {
                resultReceiver =
                    getParcelable<ResultReceiver>(Constants.KEY_RESULT_RECEIVER) as ResultReceiver
                val friendPickerParam = getParcelable<Parcelable>(Constants.KEY_FRIEND_PICKER_PARAM)
                val chatPickerParam = getParcelable<Parcelable>(Constants.KEY_CHAT_PICKER_PARAM)
                val isPopup = getBoolean(Constants.KEY_IS_POPUP)
                val param = chatPickerParam ?: friendPickerParam!!

                setViewAppearance(isPopup, param)
                setViewOrientation(param)
                LayoutUtils.initialize(applicationContext, isPopup)
            }
        } catch (e: Throwable) {
            SdkLog.e(e)
            sendError(ClientError(ClientErrorCause.Unknown).apply { initCause(e) })
        }

        if (savedInstanceState == null) {
            setFragment()
        }
    }

    override fun onBackPressed() {
        if (supportFragmentManager.backStackEntryCount == 0) {
            sendError(ClientError(ClientErrorCause.Cancelled, "cancelled."))
            Repository.clearData()
        }
        super.onBackPressed()
    }

    private fun sendError(exception: KakaoSdkError) {
        if (this::resultReceiver.isInitialized) {
            resultReceiver.send(
                Activity.RESULT_CANCELED,
                Bundle().apply {
                    putSerializable(Constants.KEY_EXCEPTION, exception)
                }
            )
        }
        finish()
    }
}