/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.view

import android.content.Context
import android.text.Editable
import android.text.TextWatcher
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import androidx.constraintlayout.widget.ConstraintLayout
import com.kakao.sdk.v2.partner.friend.R

internal class SearchBar : ConstraintLayout, TextWatcher {
    lateinit var editText: EditText
    private lateinit var deleteButton: ViewGroup
    private lateinit var placeholderMarginView: View

    private var searchAction: ((Context, String) -> Unit)? = null

    constructor(context: Context) : super(context)

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)

    constructor(context: Context, attrs: AttributeSet, defStyleAttr: Int) : super(
        context,
        attrs,
        defStyleAttr
    )

    init {
        initialize(context)
    }

    fun setPlaceholderText(placeholder: String) {
        editText.hint = placeholder
    }

    fun setSearchAction(searchAction: (Context, String) -> Unit) {
        this.searchAction = searchAction
    }

    override fun afterTextChanged(s: Editable?) {
        if (s.isNullOrEmpty()) {
            deleteButton.visibility = View.GONE
            placeholderMarginView.visibility = View.VISIBLE
        } else {
            deleteButton.visibility = View.VISIBLE
            placeholderMarginView.visibility = View.GONE
        }
        searchAction?.let { it(context, s.toString()) }
    }

    override fun beforeTextChanged(sequence: CharSequence?, start: Int, count: Int, after: Int) {
    }

    override fun onTextChanged(sequence: CharSequence?, start: Int, before: Int, count: Int) {
    }

    private fun initialize(context: Context) {
        val inflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
        val view = inflater.inflate(R.layout.kakao_sdk_search_bar, this, true)
        editText = view.findViewById(R.id.search_bar)
        deleteButton = view.findViewById(R.id.search_bar_delete_btn)
        placeholderMarginView = view.findViewById(R.id.placeholder_margin_view)

        editText.addTextChangedListener(this)
        deleteButton.setOnClickListener { editText.text.clear() }
    }
}